@@ -6,6 +6,8 @@ from immutabledict import immutabledict
 import strax
 import straxen
 
+from straxen.get_corrections import is_cmt_option
+
 export, __all__ = strax.exporter()
 
 
@@ -20,10 +22,15 @@ export, __all__ = strax.exporter()
     strax.Option('baseline_samples_nv',
                  default=('baseline_samples_nv', 'ONLINE', True), track=True,
                  help="Number of samples used in baseline rms calculation"),
-    strax.Option('hit_min_amplitude_nv',
-                 default=20, track=True,
-                 help='Minimum hit amplitude in ADC counts above baseline. '
-                      'Specify as a tuple of length n_nveto_pmts, or a number.'),
+    strax.Option(
+        'hit_min_amplitude_nv',
+        default=('hit_thresholds_nv', 'ONLINE', True), track=True,
+        help='Minimum hit amplitude in ADC counts above baseline. '
+             'Specify as a tuple of length n_nveto_pmts, or a number, '
+             'or a string like "pmt_commissioning_initial" which means calling '
+             'hitfinder_thresholds.py, '
+             'or a tuple like (correction=str, version=str, nT=boolean), '
+             'which means we are using cmt.'),
     strax.Option('n_lone_records_nv', type=int, default=2, track=False,
                  help="Number of lone hits to be stored per channel for diagnostic reasons."),
     strax.Option('channel_map', track=False, type=immutabledict,
@@ -66,6 +73,18 @@ class nVETORecorder(strax.Plugin):
             self.baseline_samples = straxen.get_correction_from_cmt(
                 self.run_id, self.config['baseline_samples_nv'])
 
+        # Check config of `hit_min_amplitude_nv` and define hit thresholds
+        # if cmt config
+        if is_cmt_option(self.config['hit_min_amplitude_nv']):
+            self.hit_thresholds = straxen.get_correction_from_cmt(self.run_id,
+                self.config['hit_min_amplitude_nv'])
+        # if hitfinder_thresholds config
+        elif isinstance(self.config['hit_min_amplitude_nv'], str):
+            self.hit_thresholds = straxen.hit_min_amplitude(
+                self.config['hit_min_amplitude_nv'])
+        else: # int or array
+            self.hit_thresholds = self.config['hit_min_amplitude_nv']
+
     def infer_dtype(self):
         self.record_length = strax.record_length_from_dtype(
             self.deps['raw_records_nv'].dtype_for('raw_records_nv'))
@@ -101,8 +120,7 @@ class nVETORecorder(strax.Plugin):
         strax.baseline(temp_records,
                        baseline_samples=self.baseline_samples,
                        flip=True)
-        hits = strax.find_hits(temp_records,
-                               min_amplitude=self.config['hit_min_amplitude_nv'])
+        hits = strax.find_hits(temp_records, min_amplitude=self.hit_thresholds)
         del temp_records
 
         # First we have to split rr into records and lone records:
