@@ -5,7 +5,8 @@ import socket
 from tqdm import tqdm
 from copy import deepcopy
 import strax
-from pprint import pprint
+from .rucio import key_to_rucio_did
+import warnings
 
 try:
     import utilix
@@ -121,6 +122,7 @@ class RunDB(strax.StorageFrontend):
                 self.available_query.append({'host': host_alias})
 
         if self.rucio_path is not None:
+            # TODO replace with rucio backend in the rucio module
             self.backends.append(strax.rucio(self.rucio_path))
             # When querying for rucio, add that it should be dali-userdisk
             self.available_query.append({'host': 'rucio-catalogue',
@@ -149,7 +151,7 @@ class RunDB(strax.StorageFrontend):
     def _find(self, key: strax.DataKey,
               write, allow_incomplete, fuzzy_for, fuzzy_for_options):
         if fuzzy_for or fuzzy_for_options:
-            raise NotImplementedError("Can't do fuzzy with RunDB yet.")
+            warnings.warn("Can't do fuzzy with RunDB yet. Only returning exact matches")
 
         # Check if the run exists
         if self.runid_field == 'name':
@@ -159,7 +161,7 @@ class RunDB(strax.StorageFrontend):
 
         # Check that we are in rucio backend
         if self.rucio_path is not None:
-            rucio_key = self.key_to_rucio_did(key)
+            rucio_key = key_to_rucio_did(key)
             rucio_available_query = self.available_query[-1]
             dq = {
                 'data': {
@@ -223,7 +225,7 @@ class RunDB(strax.StorageFrontend):
 
     def find_several(self, keys: typing.List[strax.DataKey], **kwargs):
         if kwargs.get('fuzzy_for', False) or kwargs.get('fuzzy_for_options', False):
-            raise NotImplementedError("Can't do fuzzy with RunDB yet.")
+            warnings.warn("Can't do fuzzy with RunDB yet. Only returning exact matches")
         if not len(keys):
             return []
         if not len(set([k.lineage_hash for k in keys])) == 1:
@@ -264,10 +266,9 @@ class RunDB(strax.StorageFrontend):
 
     def _list_available(self, key: strax.DataKey,
                         allow_incomplete, fuzzy_for, fuzzy_for_options):
-        if fuzzy_for or fuzzy_for_options:
-            raise NotImplementedError("Can't do fuzzy with RunDB yet.")
-        if allow_incomplete:
-            raise NotImplementedError("Can't allow_incomplete with RunDB yet")
+        if fuzzy_for or fuzzy_for_options or allow_incomplete:
+            # The RunDB frontend can do neither fuzzy nor incomplete
+            warnings.warn('RunDB cannot do fuzzy or incomplete')
 
         q = self._data_query(key)
         q.update(self.number_query())
@@ -323,8 +324,3 @@ class RunDB(strax.StorageFrontend):
         if q_number:
             return {'number': q_number}
         return {}
-
-    @staticmethod
-    def key_to_rucio_did(key: strax.DataKey):
-        """Convert a strax.datakey to a rucio did field in rundoc"""
-        return f'xnt_{key.run_id}:{key.data_type}-{key.lineage_hash}'
